package org.zeroturnaround.javarebel.integration.servlet;

import org.zeroturnaround.javarebel.ClassResourceSource;
import org.zeroturnaround.javarebel.IntegrationFactory;
import org.zeroturnaround.javarebel.Plugin;
import org.zeroturnaround.javarebel.integration.generic.BaseServletContextImplClassBytecodeProcessor;
import org.zeroturnaround.javarebel.integration.generic.ServletContextImplClassBytecodeProcessor;
import org.zeroturnaround.javarebel.integration.servlet.cbp.ServletContextCBP;
import org.zeroturnaround.javarebel.integration.servlet.cbp.jetty5.JettyServletHandlerCBP;
import org.zeroturnaround.javarebel.integration.servlet.cbp.jetty6.JettyContextHandlerCBP;
import org.zeroturnaround.javarebel.integration.servlet.cbp.resin.ResinContextHandlerCBP;
import org.zeroturnaround.javarebel.integration.servlet.cbp.tomcat.WebappLoaderCBP;

/**
 * @author Rein Raudjärv
 */
public class ServletIntegrationPlugin implements Plugin {
  public void preinit() {
    ClassLoader cl = getClass().getClassLoader();

    // ServletContext
    IntegrationFactory.getInstance().addIntegrationProcessor(
        cl,
        "javax.servlet.ServletContext", 
        new ServletContextCBP());

    // JETTY 5.x, 6.x

    // Register servlet context
    IntegrationFactory.getInstance().addIntegrationProcessor(
        "org.mortbay.jetty.servlet.ServletHandler", // Since at least Jetty 5
        new JettyServletHandlerCBP());
    IntegrationFactory.getInstance().addIntegrationProcessor(
        "org.mortbay.jetty.handler.ContextHandler", // Since Jetty 6
        new JettyContextHandlerCBP());

    // Delegate resource providing to JavaRebel
    IntegrationFactory.getInstance().addIntegrationProcessor(
        new String[] {
            "org.mortbay.jetty.servlet.ServletHandler$Context", // Up to Jetty 5 (Removed since Jetty 6)
            "org.mortbay.jetty.handler.ContextHandler$SContext" // Since Jetty 6 
        },
        new BaseServletContextImplClassBytecodeProcessor());

    /*
     * TOMCAT 5.5
     * GLASSFISH 2.1 (Tested on 2.1-b60e)
     * JBOSS 4.2, 5.0 (Tested on 4.2.2 GA, 5.0.0 GA)
     */

    IntegrationFactory.getInstance().addIntegrationProcessor(
        "org.apache.catalina.core.ApplicationContext", 
        new ServletContextImplClassBytecodeProcessor());
    IntegrationFactory.getInstance().addIntegrationProcessor(
        "org.apache.catalina.loader.WebappLoader", 
        new WebappLoaderCBP());

    // WEBLOGIC 9.2

    IntegrationFactory.getInstance().addIntegrationProcessor(
        "weblogic.servlet.internal.WebAppServletContext", 
        new ServletContextImplClassBytecodeProcessor());

    // OC4J 9.x, 10.x (Tested on 9.0.4, 10.1.2.0.2, 10.1.3.3.0, 10.1.3.4.0)

    IntegrationFactory.getInstance().addIntegrationProcessor(
        "com.evermind.server.http.HttpApplication", 
        new ServletContextImplClassBytecodeProcessor());
    
    // IBM WAS 6.1 (Tested on 6.1.0.0)

    IntegrationFactory.getInstance().addIntegrationProcessor(
        "com.ibm.ws.webcontainer.facade.ServletContextFacade",
        new ServletContextImplClassBytecodeProcessor());
    
    // CAUCHO RESIN (Tested on 3.0.24)
    IntegrationFactory.getInstance().addIntegrationProcessor(
        "com.caucho.server.webapp.Application",
        new ResinContextHandlerCBP());
  }

  public boolean checkDependencies(ClassLoader cl, ClassResourceSource crs) {
    return crs.getClassResource("javax.servlet.ServletContext") != null;
  }
 
  public String getId() {
    return "servlet_integration_plugin";
  }
  
  public String getName() {
    return null;
  }
  
  public String getDescription() {
    return null;
  }  
}
